﻿// Copyright (c) 2011 Andrea Martinelli
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following
// conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.


using System;
using System.IO;

namespace Antiufo.HttpPost
{

    internal class StreamFormData : FormData, IDisposable
    {
        public StreamFormData(Stream sourceStream, string filename)
        {
            this.sourceStream = sourceStream;
            this.filename = filename;
        }

        public StreamFormData(Action<Stream> writer, string filename)
        {
            this.writer = writer;
            this.filename = filename;
        }

        internal readonly Action<Stream> writer;
        internal readonly Stream sourceStream;
        private string filename;

        protected override byte[] GetMultipartHeader()
        {
            return System.Text.Encoding.UTF8.GetBytes(" filename=\"C:\\fakepath\\" + filename + "\"\r\n" +
    "Content-Type: application/octet-stream\r\n\r\n");
        }

        public override long InternalGetLength()
        {
            if (sourceStream == null) throw new NotSupportedException();
            return sourceStream.Length;
        }

        protected override void InternalWriteTo(Stream stream)
        {
            if (writer != null)
            {
                writer(stream);
            }
            else
            {
                var buffer = new byte[1024];
                var bytesRead = 0;
                while ((bytesRead = sourceStream.Read(buffer, 0, buffer.Length)) != 0)
                {
                    stream.Write(buffer, 0, bytesRead);
                }
            }
            Dispose();
        }

        public void Dispose()
        {
            if (sourceStream != null)
                sourceStream.Dispose();
        }
    }
}
